//---------------------------------------------------------------------------
// Copyright(c) 1999 arton
//
#include <vcl.h>
#pragma hdrstop

#include "Main.h"
#include "MACCAPLib_TLB.h"
#include "Proto.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
static const char* ICMPTypes[] = {
		"Echo Reply",//                              [RFC792]
		"Unassigned",//                              [JBP]
		"Unassigned",//                              [JBP]
		"Destination Unreachable",//                 [RFC792]
		"Source Quench",//                           [RFC792]
		"Redirect (chage route)", //                 [RFC792]
		"Alternate Host Address",//                  [JBP]
		"Unassigned",//                              [JBP]
		"Echo Request",//                            [RFC792]
		"Router Advertisement",//                    [RFC1256]
		"Router Selection",//                        [RFC1256]
		"Time Exceeded",//                           [RFC792]
		"Parameter Problem",//                       [RFC792]
		"Timestamp",//                               [RFC792]
		"Timestamp Reply",//                         [RFC792]
		"Information Request",//                     [RFC792]
		"Information Reply",//                       [RFC792]
		"Address Mask Request",//                    [RFC950]
		"Address Mask Reply",//                      [RFC950]
		"Reserved (for Security)",//                 [Solo]
		"Reserved (for Robustness Experiment)",//    [ZSu]
		"Reserved (for Robustness Experiment)",//    [ZSu]
		"Reserved (for Robustness Experiment)",//    [ZSu]
		"Reserved (for Robustness Experiment)",//    [ZSu]
		"Reserved (for Robustness Experiment)",//    [ZSu]
		"Reserved (for Robustness Experiment)",//    [ZSu]
		"Reserved (for Robustness Experiment)",//    [ZSu]
		"Reserved (for Robustness Experiment)",//    [ZSu]
		"Reserved (for Robustness Experiment)",//    [ZSu]
		"Reserved (for Robustness Experiment)",//    [ZSu]
		"Traceroute",//                              [RFC1393]
		"Datagram Conversion Error",//               [RFC1475]
		"Mobile Host Redirect",//              [David Johnson]
		"IPv6 Where-Are-You",//                 [Bill Simpson]
		"IPv6 I-Am-Here",//                     [Bill Simpson]
		"Mobile Registration Request",//        [Bill Simpson]
		"Mobile Registration Reply",//          [Bill Simpson]
};

LPCSTR TMainForm::GetICMPTypeName(BYTE b)
{
	if (b <= COUNTOF(ICMPTypes))
		return ICMPTypes[b];
	else
		return "";
}
//---------------------------------------------------------------------------
int TMainForm::PrintICMP(TStrings* ss, LPBYTE pb, int len)
{
	static const char* UnreachableCodes[] = {
		"Network Unreachable",
		"Host Unreachable",
		"Protocol Unreachable",
		"Port Unreachable",
		"Fragmentation Needed and DF set",
		"Source Route Failed",
		"Destination network unknown",
		"Destination host unknown",
		"Source host isolated",
		"Communication with destination network administratively prohibited",
		"Communication with destination host Administratively prohibited",
		"Network unreachable for type of service",
		"Host unreachable for type of service",
	};
	static const char* ChangeRouteCodes[] = {
		"Redirect datagrams for the Network (obsoleted)",
		"Redirect datagrams for the Host",
		"Redirect datagrams for the Type of Service and Network",
		"Redirect datagrams for the Type of Service and Host",
	};
	char sz[128];
	wsprintf(sz, "ICMP: %02d: ", *pb);
	AnsiString as(sz);
	ss->Append(as + GetICMPTypeName(*pb));
	switch (*pb)
	{
	case 8:
	case 0:
		// Echo
		wsprintf(sz, "ICMP:   Checksum: %d", ntohs(pb + 2));
		ss->Append(sz);
		wsprintf(sz, "ICMP:         ID: %04X, sequence: %d", ntohs(pb + 4), ntohs(pb + 6));
		ss->Append(sz);
		wsprintf(sz, "optional data: (%dbytes)", len - 8);
		ss->Append(sz);
		if ((len - 8) > 0)
			DumpData(pb + 8, len - 8, 0);
		break;
	case 3:
		// Unreachable
		wsprintf(sz, "ICMP:   Code: %2d (", *(pb + 1));
		as = sz;
		if (*(pb + 1) >= COUNTOF(UnreachableCodes))
			as += "Unknown Code";
		else
			as += UnreachableCodes[*(pb + 1)];
		as += ")";
		ss->Append(as);
		// fall down to the next case
	case 4:
		// Source Quench
		wsprintf(sz, "ICMP:   Checksum: %d", ntohs(pb + 2));
		ss->Append(sz);
		ss->Append("\r\nDatagram:");
		if (len - 8 > 0)
			DumpData(pb + 8, len - 8, 0);
		break;
	case 5:
		// route change
		wsprintf(sz, "ICMP:   Code: %2d (", *(pb + 1));
		as = sz;
		if (*(pb + 1) >= COUNTOF(ChangeRouteCodes))
			as += "Unknown Code";
		else
			as += ChangeRouteCodes[*(pb + 1)];
		as += ")";
		ss->Append(as);
		wsprintf(sz, "ICMP:   Checksum: %d", ntohs(pb + 2));
		ss->Append(sz);
		wsprintf(sz, "ICMP:   Gateway IP Address: %d.%d.%d.%d", *(pb + 4), *(pb + 5), *(pb + 6), *(pb + 7));
		ss->Append(sz);
		ss->Append("\r\nDatagram:");
		if (len - 8 > 0)
			DumpData(pb + 8, len - 8, 0);
		break;
	case 11:
		// Detecting Circular
		if (*(pb + 1) == 0)
			ss->Append("ICMP:   Code: 0 (Time-to-live count exceeded)");
		else if (*(pb + 1) == 1)
			ss->Append("ICMP:   Code: 1 (Fragment reassembly time exceeded)");
		else
		{
			wsprintf(sz, "ICMP:   Code: %2d (Unknown)", *(pb + 1));
			ss->Append(sz);
		}
		wsprintf(sz, "ICMP:   Checksum: %d", ntohs(pb + 2));
		ss->Append(sz);
		ss->Append("\r\nDatagram:");
		if (len - 8 > 0)
			DumpData(pb + 8, len - 8, 0);
		break;
	case 12:
		// Other problem
		switch (*(pb + 1))
		{
		case 0:
			wsprintf(sz, "ICMP:   Code: 0, Pointer=%d(0x%02X) indicates the error", *(pb + 4), *(pb + 4));
			ss->Append(sz);
			break;
		case 1:
			ss->Append("ICMP:   Code: 1 (Missing a Required Option)");
			break;
		case 2:
			ss->Append("ICMP:   Code: 2 (Bad Length)");
			break;
		default:
			wsprintf(sz, "ICMP:   Code: %2d (Unknown, pointer=%d(0x%02X)", *(pb + 1), *(pb + 4), *(pb + 4));
			ss->Append(sz);
			break;
		}
		wsprintf(sz, "ICMP:   Checksum: %d", ntohs(pb + 2));
		ss->Append(sz);
		ss->Append("\r\nDatagram:");
		if (len - 8 > 0)
			DumpData(pb + 8, len - 8, 0);
		break;
	case 13:
	case 14:
		// Clock Synch
		wsprintf(sz, "ICMP:            Checksum: %d", ntohs(pb + 2));
		ss->Append(sz);
		wsprintf(sz, "ICMP:                  ID: %04X, sequence: %d", ntohs(pb + 4), ntohs(pb + 6));
		ss->Append(sz);
		wsprintf(sz, "ICMP: originate timestamp: %10d(%08X)", ntohl(pb + 8), ntohl(pb + 8));
		ss->Append(sz);
		if (*pb == 14)
		{
			wsprintf(sz, "ICMP:   receive timestamp: %10d(%08X)", ntohl(pb + 12), ntohl(pb + 12));
			ss->Append(sz);
			wsprintf(sz, "ICMP:  transmit timestamp: %10d(%08X)", ntohl(pb + 16), ntohl(pb + 16));
			ss->Append(sz);
		}
		break;
	case 17:
		// Obtaining Subnet request
		wsprintf(sz, "ICMP:   Checksum: %d", ntohs(pb + 2));
		ss->Append(sz);
		break;
	case 18:
		// Obtaining Subnet reply
		wsprintf(sz, "ICMP:      Checksum: %d", ntohs(pb + 2));
		ss->Append(sz);
		wsprintf(sz, "ICMP:  ADDRESS MASK: %d.%d.%d.%d", *(pb + 8), *(pb + 9), *(pb + 10), *(pb + 11));
		ss->Append(sz);
		break;
	default:
		wsprintf(sz, "ICMP:       code: %d", *(pb + 1));
		ss->Append(sz);
		wsprintf(sz, "ICMP:   Checksum: %d", ntohs(pb + 2));
		ss->Append(sz);
		len = 4;
		break;
	}
	return len;
}
//---------------------------------------------------------------------------
AnsiString TMainForm::GetUDPInfo(LPBYTE pb)
{
	char sz[128];
	short ds = ntohs(pb + sizeof(short));
	short ss = ntohs(pb);
	LPCTSTR ps = GetServiceName(ss, false);
	LPCTSTR pd = GetServiceName(ds, false);
	wsprintf(sz, "%d-%d", ss, ds);
	AnsiString as(sz);
	if (ps || pd)
	{
		as += "(";
		as += (ps) ? ps : pd;
		as += ")";
	}
	return as;
}
//---------------------------------------------------------------------------
int TMainForm::PrintUDP(TStrings* ss, LPBYTE pb, int len)
{
	char sz[128];
	LPCSTR ps = GetServiceName(ntohs(pb), false);
	wsprintf(sz, "UDP:      Source Port: %5d (%s)", ntohs(pb), (ps) ? ps : " - ");
	ss->Append(sz);
	ps = GetServiceName(ntohs(pb + 2), false);
	wsprintf(sz, "UDP: Destination Port: %5d (%s)", ntohs(pb + 2), (ps) ? ps : " - ");
	ss->Append(sz);
	wsprintf(sz, "UDP:         Checksum: %d", ntohs(pb + 6));
	ss->Append(sz);
	return 8;
}
//---------------------------------------------------------------------------
AnsiString TMainForm::GetTCPInfo(LPBYTE pb)
{
	char sz[64];
	short ds = ntohs(pb + sizeof(short));
	short ss = ntohs(pb);
	LPCTSTR ps = GetServiceName(ss, true);
	LPCTSTR pd = GetServiceName(ds, true);
	wsprintf(sz, "%d-%d", ss, ds);
	AnsiString as(sz);
	if (ps || pd)
	{
		as += "(";
		as += (ps) ? ps : pd;
		as += ") ";
	}
	else
	{
		as += " ";
	}
	BYTE b = *(pb + 13);
	if (b & TCP_SYN)
		as += "SYN ";
	if (b & TCP_RST)
		as += "RST ";
	if (b & TCP_FIN)
		as += "FIN ";
	if (b & TCP_PSH)
		as += "PSH ";
	if (b & TCP_URG)
		as += "URG ";
	if (b & TCP_ACK)
		as += "ACK ";
	return as.TrimRight();
}
//---------------------------------------------------------------------------
int TMainForm::PrintTCP(TStrings* ss, LPBYTE pb, int len)
{
	char sz[128];
	LPCSTR ps = GetServiceName(ntohs(pb), true);
	wsprintf(sz, "TCP:      Source Port: %5d (%s)", ntohs(pb), (ps) ? ps : " - ");
	ss->Append(sz);
	ps = GetServiceName(ntohs(pb + 2), true);
	wsprintf(sz, "TCP: Destination Port: %5d (%s)", ntohs(pb + 2), (ps) ? ps : " - ");
	ss->Append(sz);
	wsprintf(sz, "TCP:         Sequence Number: %10lu", ntohl(pb + 4));
	ss->Append(sz);
	wsprintf(sz, "TCP:  Acknowledgement Number: %10lu", ntohl(pb + 8));
	ss->Append(sz);
	int hlen = (*(pb + 12) >> 4) * sizeof(long);
	wsprintf(sz, "TCP:           Header Length: %10d", hlen);
	ss->Append(sz);
	ss->Append("TCP: \r\nTCP: Code Bits");
	BYTE b = *(pb + 13);
	wsprintf(sz, "TCP:                     URG: %d", (b >> 5) & 1);
	ss->Append(sz);
	wsprintf(sz, "TCP:                     ACK: %d", (b >> 4) & 1);
	ss->Append(sz);
	wsprintf(sz, "TCP:                     PSH: %d", (b >> 3) & 1);
	ss->Append(sz);
	wsprintf(sz, "TCP:                     RST: %d", (b >> 2) & 1);
	ss->Append(sz);
	wsprintf(sz, "TCP:                     SYN: %d", (b >> 1) & 1);
	ss->Append(sz);
	wsprintf(sz, "TCP:                     FIN: %d\r\nTCP:", b & 1);
	ss->Append(sz);
	wsprintf(sz, "TCP:                  Window: %5d", ntohs(pb + 14));
	ss->Append(sz);
	wsprintf(sz, "TCP:                Checksum: %5d", ntohs(pb + 16));
	ss->Append(sz);
	wsprintf(sz, "TCP:          Urgent Pointer: %5d", ntohs(pb + 18));
	ss->Append(sz);

	if (hlen > 20)
	{
		wsprintf(sz, "TCP: \r\nTCP: Options: (%dbytes)", hlen - 20);
		ss->Append(sz);
		DumpData(pb + 20, hlen - 20, 0);
	}
	return hlen;
}
//---------------------------------------------------------------------------
const char* TMainForm::GetServiceName(short port, bool tcp)
{
	struct ServiceEntry {
		const char* ServiceName;
		short Port;
	};
	static const ServiceEntry TCPServices[] = {
		{ "tcpmux", 1, },
		{ "echo", 7, },
		{ "discard", 9, },
		{ "systat",	11, },
		{ "daytime", 13, },
		{ "netstat",	15, },
		{ "qotd",	17, },
		{ "chargen",	19, },
		{ "ftp-data",	20, },
		{ "ftp",		21, },
		{ "telnet",		23, },
		{ "smtp",		25, },
		{ "time",		37, },
		{ "whois",		43, },
		{ "domain",		53, },
		{ "mtp",             57, },
		{ "gopher",		70, },
		{ "rje",		77, },
		{ "finger",		79, },
		{ "http",		80, },
		{ "link",		87, },
		{ "kerberos",	88, },
		{ "supdup",		95, },
		{ "linuxconf",	98, },
		{ "hostnames",	101, },
		{ "iso-tsap",	102, },
		{ "x400",		103, },
		{ "x400-snd",	104, },
		{ "csnet-ns",	105, },
		{ "pop-2",		109, },
		{ "pop-3",		110, },
		{ "sunrpc",		111, },
		{ "auth",		113, },
		{ "sftp",		115, },
		{ "uucp-path",	117, },
		{ "nntp",       119, },
		{ "ntp",		123, },
		{ "netbios-ns",      137, },
		{ "netbios-dgm",     138, },
		{ "netbios-ssn",     139, },
		{ "imap", 		143, },
		{ "NeWS",		144, },
		{ "exec",		512, },
		{ "login",		513, },
		{ "shell",		514, },
		{ "printer",	515, },
		{ "efs",        520, },
		{ "tempo",      526, },
		{ "courier",	530, },
		{ "conference", 531, },
		{ "netnews",    532, },
		{ "uucp",		540, },
		{ "klogin",		543, },
		{ "kshell",		544, },
		{ "remotefs",   556, },
		{ "pcserver",	600, },
		{ "kerberos-adm",	749, },
		{ "kerberos-sec",	750, },
		{ "kerberos_master",	751, },
		{ "krb5_prop",	754, },
		{ "listen",		1025, },
		{ "nterm",		1026, },
		{ "kpop",		1109, },
		{ "ingreslock",      1524, },
		{ "tnet",            1600, },
		{ "cfinger",		2003, },
		{ "eklogin",		2105, },
		{ "krb524",		4444, },
		{ "irc",		6667, },
		{ "dos",		7000, },
		{ "http-proxy",	8080, },
	};
	static const ServiceEntry UDPServices[] = {
		{ "echo",		7, },
		{ "discard",	9, },
		{ "daytime",	13, },
		{ "chargen",	19, },
		{ "time",		37, },
		{ "rlp",		39, },
		{ "name",		42, },
		{ "domain",		53, },
		{ "bootps",		67, },
		{ "bootpc",		68, },
		{ "tftp",		69, },
		{ "kerberos",	88, },
		{ "sunrpc",		111, },
		{ "ntp",		123, },
		{ "netbios-ns",      137, },
		{ "netbios-dgm",     138, },
		{ "snmp",		161, },
		{ "snmp-trap",	162, },
		{ "biff",		512, },
		{ "who",		513, },
		{ "syslog",		514, },
		{ "talk",		517, },
		{ "ntalk",		518, },
		{ "route",		520, },
		{ "timed",		525, },
		{ "new-rwho",	550, },
		{ "rmonitor",	560, },
		{ "monitor",		561, },
		{ "netwall",         533, },
		{ "mount",		635, },
		{ "pcnfs",		640, },
		{ "bwnfs",		650, },
		{ "kerberos-adm",	749, },
		{ "kerberos-sec",	750, },
		{ "kerberos_master",	751, },
		{ "nfs",		2049, },
	};

	const ServiceEntry* pEntry = (tcp) ? TCPServices : UDPServices;
	int r = (tcp) ? COUNTOF(TCPServices) : COUNTOF(UDPServices);
	r--;
	int l = 0;
	int m;
	while (l < r)
	{
		m = (l + r) / 2;
		if ((pEntry + m)->Port == port)
			return (pEntry + m)->ServiceName;
		else if ((pEntry + m)->Port < port)
			l = m + 1;
		else
			r = m;
	}
	if ((pEntry + l)->Port == port)
		return (pEntry + l)->ServiceName;
	return NULL;
}
//---------------------------------------------------------------------------
static const char* OSPFTypes[] = {
	"Unknown",
	"Hello",
	"Database Description",
	"Link State Request",
	"Link State Update",
	"Link State Acknowledgment",
};

AnsiString TMainForm::GetOSPFInfo(LPBYTE pb)
{
	BYTE b = *(pb + 1);
	if (b >= COUNTOF(OSPFTypes))
		return "Unknown";
	else
		return OSPFTypes[b];
}
//---------------------------------------------------------------------------
int TMainForm::PrintOSPF(TStrings* ss, LPBYTE pb, int len)
{
	// RFC 1247
/*
	0                   1                   2                   3
	0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |   Version #   |     Type      |         Packet length         |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |                          Router ID                            |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |                           Area ID                             |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |           Checksum            |             Autype            |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |                       Authentication                          |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |                       Authentication                          |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
*/
	static const char* OSPFAuthTypes[] = {
		"No Authentication",
		"Simple Password",
	};
	char sz[128];
	if (*(pb + 1) >= COUNTOF(OSPFTypes))
		wsprintf(sz, "OSPF:         Version: %d  Type: %d (Unknown)", *pb, *(pb + 1));
	else
		wsprintf(sz, "OSPF:         Version: %d  Type: %d (%s)", *pb, *(pb + 1), OSPFTypes[*(pb + 1)]);
	ss->Append(sz);
	wsprintf(sz, "OSPF:   Packet length: %d", ntohs(pb + 2));
	ss->Append(sz);
	wsprintf(sz, "OSPF:       Router ID: %d.%d.%d.%d", *(pb + 4), *(pb + 5), *(pb + 6), *(pb + 7));
	ss->Append(sz);
	wsprintf(sz, "OSPF:         Area ID: %d", ntohl(pb + 8));
	ss->Append(sz);
	wsprintf(sz, "OSPF:        Checksum: %5d", ntohs(pb + 12));
	ss->Append(sz);
	short s = ntohs(pb + 14);
	if (s >= COUNTOF(OSPFAuthTypes))
		wsprintf(sz, "OSPF:  Authentication: %d(Reserved for assignment by the IANA)", s);
	else
		wsprintf(sz, "OSPF:  Authentication: %d(%s)", s, OSPFAuthTypes[s]);
	ss->Append(sz);
	ss->Append("OSPF: Authentication Data:");
	DumpData(pb + 16, sizeof(long) * 2, 0);

	return OSPF_HLEN;
}
//---------------------------------------------------------------------------
