// Copyright(c) 1999 arton

#include "stdafx.h"
#include "MacCapv2.h"
#include "Adapters.h"
#include "NCap.h"

// This class must be a singleton.
// Because, the device driver (NCapDD) is loaded by it.
// 
// 1. In FinalConstruct() it loads NCapDD.
//	  It's thread safe because ATL keeps FinalConstruct() in exclusive state.
// 2. If NCapDD was already loaded, it causes an error.
// 3. holds driver's handle.
// 4. In FinalRelease() it unloads NCapDD.
//
static const WCHAR scDevPrefix[] = L"\\Device\\NCapDD_";
/////////////////////////////////////////////////////////////////////////////
// CAdapters
HRESULT STDMETHODCALLTYPE CAdapters::get__NewEnum( 
            /* [retval][out] */ IUnknown __RPC_FAR *__RPC_FAR *ppenum)
{
	if (!ppenum) return E_POINTER;
	*ppenum = NULL;
	typedef CComObject<CComEnumOnSTL<IEnumVARIANT, &IID_IEnumVARIANT, VARIANT, _Copy<VARIANT>, ListVars > > enumvar;
	enumvar* p = NULL;
	ATLTRY(p = new enumvar);
	if(p == NULL)
		return E_OUTOFMEMORY;
	HRESULT hRes = p->Init(GetControllingUnknown(), m_listAdapters);
	if (SUCCEEDED(hRes))
		hRes = p->QueryInterface(IID_IEnumVARIANT, (void**)ppenum);
	if (FAILED(hRes))
		delete p;
	return hRes;
}
        
HRESULT STDMETHODCALLTYPE CAdapters::get_Length( 
            /* [retval][out] */ long __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	*pResult = m_aNIC.size();
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CAdapters::get_Item( 
            /* [in] */ long index,
            /* [retval][out] */ BSTR __RPC_FAR *pRetVal)
{
	if (!pRetVal) return E_POINTER;
	if (index < 0 || index >= m_aNIC.size()) return DISP_E_BADINDEX;
	*pRetVal = SysAllocString(m_aNIC[index].m_sCaption.c_str());
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CAdapters::get_MACAddress( 
            /* [in] */ long index,
            /* [retval][out] */ VARIANT __RPC_FAR *pRetVal)
{
	if (!pRetVal) return E_POINTER;
	if (index < 0 || index >= m_aNIC.size()) return DISP_E_BADINDEX;
	SAFEARRAYBOUND rgsabound[1];
	rgsabound[0].cElements = 6;
	rgsabound[0].lLbound = 0;
	SAFEARRAY* pAry = SafeArrayCreate(VT_UI1, 1, rgsabound);
	if (pAry == NULL) return E_OUTOFMEMORY;
	SafeArrayLock(pAry);
	CopyMemory(pAry->pvData, m_aNIC[index].m_bMACAddress, 6);
	SafeArrayUnlock(pAry);
	VariantInit(pRetVal);
	pRetVal->vt = VT_UI1 | VT_ARRAY;
	pRetVal->parray = pAry;
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CAdapters::get_VendorName( 
            /* [in] */ long index,
            /* [retval][out] */ BSTR __RPC_FAR *pRetVal)
{
	if (!pRetVal) return E_POINTER;
	if (index < 0 || index >= m_aNIC.size()) return DISP_E_BADINDEX;
	*pRetVal = SysAllocString(m_aNIC[index].m_sManufacturer.c_str());
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CAdapters::get_ServiceName( 
            /* [in] */ long index,
            /* [retval][out] */ BSTR __RPC_FAR *pRetVal)
{
	if (!pRetVal) return E_POINTER;
	if (index < 0 || index >= m_aNIC.size()) return DISP_E_BADINDEX;
	*pRetVal = SysAllocString(m_aNIC[index].m_sServiceName.c_str());
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CAdapters::get_Description( 
            /* [in] */ long index,
            /* [retval][out] */ BSTR __RPC_FAR *pRetVal)
{
	if (!pRetVal) return E_POINTER;
	if (index < 0 || index >= m_aNIC.size()) return DISP_E_BADINDEX;
	*pRetVal = SysAllocString(m_aNIC[index].m_sDescription.c_str());
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CAdapters::get_AdapterName( 
            /* [in] */ long index,
            /* [retval][out] */ BSTR __RPC_FAR *pRetVal)
{
	if (!pRetVal) return E_POINTER;
	if (index < 0 || index >= m_aNIC.size()) return DISP_E_BADINDEX;
	*pRetVal = SysAllocString(m_aNIC[index].m_sProductName.c_str());
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CAdapters::get_DeviceName( 
            /* [in] */ long index,
            /* [retval][out] */ BSTR __RPC_FAR *pRetVal)
{
	if (!pRetVal) return E_POINTER;
	if (index < 0 || index >= m_aNIC.size()) return DISP_E_BADINDEX;
	VarsIter vi = m_listAdapters.begin();
	for (int i = 0; i < index; i++) vi++;
	*pRetVal = SysAllocString((*vi).bstrVal);
	return S_OK;
}

HRESULT CAdapters::FinalConstruct()
{
	// Enumerate all services to find NDIS driver.
	HRESULT hr = FillDriver();
	if (hr != S_OK) return hr;

	// Set Bind and Export Params for NCapDD (It's not dynamic binding, only for helping NCapDD)
	hr = CreateBindParam();
	if (hr != S_OK) return hr;

	// Then Load NCapDD
	hr = LoadDriver();
	if (hr != S_OK) return hr;

	// Fetch All MAC Address
	ReadMACAddress();

	return CoCreateFreeThreadedMarshaler(
		GetControllingUnknown(), &m_pUnkMarshaler.p);
}

void CAdapters::FinalRelease()
{
	for (VarsIter it = m_listAdapters.begin(); it != m_listAdapters.end(); it++)
		VariantClear(&(*it));
	m_listAdapters.clear();

	if (m_hNCap != INVALID_HANDLE_VALUE) 
		CloseHandle(m_hNCap);
	if (m_hSrv) 
	{
	    SERVICE_STATUS  serviceStatus;
		SC_HANDLE stopHandle = OpenServiceW(m_hScm, L"NCapDD", SERVICE_ALL_ACCESS);
		if (stopHandle)
		{
			ControlService(stopHandle, SERVICE_CONTROL_STOP, &serviceStatus);
			CloseServiceHandle(stopHandle);
		}
		CloseServiceHandle(m_hSrv);
		SC_HANDLE removeHandle = OpenServiceW(m_hScm, L"NCapDD", SERVICE_ALL_ACCESS);
		if (removeHandle)
		{
			DeleteService(removeHandle);
			CloseServiceHandle(removeHandle);
		}
	}
	if (m_hScm)
		CloseServiceHandle(m_hScm);
	m_pUnkMarshaler.Release();
}


STDMETHODIMP CAdapters::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IAdapterCollectionEx,
		&IID_IAdapterCollection,
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (::InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////////
// Initialize Funcs
HRESULT CAdapters::FillDriver()
{
	HKEY hKeyCards;
	DWORD dw, dwType;
	if (RegOpenKeyExW(HKEY_LOCAL_MACHINE, 
					  L"SOFTWARE\\MicroSoft\\Windows NT\\CurrentVersion\\NetworkCards",
					  0, KEY_READ, &hKeyCards) == ERROR_SUCCESS)
	{
		WCHAR sz[256];		// I don't believe that the services have such long name.
		FILETIME ftime;
		HKEY hCard;
		for (int i = 0;; i++)
		{
			dw = sizeof(sz) / sizeof(WCHAR);
			long lResult = RegEnumKeyExW(hKeyCards, i, sz, &dw, NULL, NULL, NULL, &ftime);
			if (lResult == ERROR_NO_MORE_ITEMS) break;
			else if (lResult == ERROR_SUCCESS)
			{
				if (RegOpenKeyEx(hKeyCards, sz, 0, KEY_READ, &hCard) == ERROR_SUCCESS)
				{
					dw = sizeof(sz) / sizeof(WCHAR);
					if (RegQueryValueEx(hCard, L"ServiceName", NULL, &dwType, reinterpret_cast<LPBYTE>(sz), &dw) == ERROR_SUCCESS
						&& dwType == REG_SZ && dw > 0)
					{
						WCHAR szUpr[64];
						wcscpy(szUpr, sz);
						_wcsupr(szUpr);
						if (wcsncmp(szUpr, L"NDISWAN", 7) && wcsncmp(szUpr, L"ASYNCMAC", 8))
						{
							CAdapterInfo infoHoler;
							m_aNIC.push_back(infoHoler);	// Allocate new AdapterInfo
							CAdapterInfo& info = m_aNIC.back();
							info.m_sServiceName = sz;
							dw = sizeof(sz) / sizeof(WCHAR);
							if (RegQueryValueEx(hCard, L"Manufacturer", NULL, NULL, reinterpret_cast<LPBYTE>(sz), &dw) == ERROR_SUCCESS)
								info.m_sManufacturer = sz;
							dw = sizeof(sz) / sizeof(WCHAR);
							if (RegQueryValueEx(hCard, L"ProductName", NULL, NULL, reinterpret_cast<LPBYTE>(sz), &dw) == ERROR_SUCCESS)
								info.m_sProductName = sz;
							dw = sizeof(sz) / sizeof(WCHAR);
							if (RegQueryValueEx(hCard, L"Description", NULL, NULL, reinterpret_cast<LPBYTE>(sz), &dw) == ERROR_SUCCESS)
								info.m_sDescription = sz;
							dw = sizeof(sz) / sizeof(WCHAR);
							if (RegQueryValueEx(hCard, L"Title", NULL, NULL, reinterpret_cast<LPBYTE>(sz), &dw) == ERROR_SUCCESS)
								info.m_sCaption = sz;
						}
					}
					RegCloseKey(hCard);
				}
			}
		}
		RegCloseKey(hKeyCards);
	}
	if (m_aNIC.size() <= 0)
	{
		return MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NULL, IDS_E_NONIC);
	}
	return S_OK;
}

HRESULT CAdapters::CreateBindParam()
{
	HKEY hKeyNCap;
	DWORD dw;
	long lResult = RegCreateKeyExW(HKEY_LOCAL_MACHINE, L"SYSTEM\\CurrentControlSet\\Services\\NCapDD\\Linkage", 
		0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKeyNCap, &dw);
	if (lResult != ERROR_SUCCESS)
	{
		return MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NULL, IDS_E_REGISTRY);
	}
	dw = 1;	// 1 for the NULL
	int cbNIC(m_aNIC.size());
	for (int i = 0; i < cbNIC; i++)
	{
		dw += m_aNIC[i].m_sServiceName.length() + sizeof(scDevPrefix) / sizeof(WCHAR);	// separator NULL conatined
	}
	dw *= sizeof(WCHAR);
	LPWSTR psz = reinterpret_cast<LPWSTR>(_alloca(dw));
	LPWSTR ptemp = psz;
	for (i = 0; i < cbNIC; i++)
	{
		dw = wsprintfW(ptemp, L"\\Device\\%s", m_aNIC[i].m_sServiceName.c_str());
		m_aNIC[i].m_sDeviceName = psz;
		ptemp += dw + 1;
	}
	*ptemp++ = L'\0';
	RegSetValueExW(hKeyNCap, L"Bind", 0, REG_MULTI_SZ, reinterpret_cast<LPBYTE>(psz), (ptemp - psz) * sizeof(WCHAR));
	ptemp = psz;
	for (i = 0; i < cbNIC; i++)
	{
		dw = wsprintfW(ptemp, L"%s%s", scDevPrefix, m_aNIC[i].m_sServiceName.c_str());
		VARIANT v;
		VariantInit(&v);
		v.vt = VT_BSTR;
		v.bstrVal = SysAllocString(ptemp);
		m_listAdapters.push_back(v);
		ptemp += dw + 1;
	}
	*ptemp++ = L'\0';
	RegSetValueExW(hKeyNCap, L"Export", 0, REG_MULTI_SZ, reinterpret_cast<LPBYTE>(psz), (ptemp - psz) * sizeof(WCHAR));
	RegCloseKey(hKeyNCap);

	return S_OK;
}

HRESULT CAdapters::LoadDriver()
{
	WCHAR szDriverPath[_MAX_PATH];
	DWORD dw = GetModuleFileNameW(_Module.GetModuleInstance(), szDriverPath, _MAX_PATH);
	for (int i = dw - 1; i > 0; i--)
	{
		if (szDriverPath[i] == L'\\')
		{
			lstrcpyW(&szDriverPath[i + 1], L"NCapDD.Sys");
			break;
		}
	}
	if (i == 0 || GetFileAttributesW(szDriverPath) == 0xffffffff) 
	{
		return MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NULL, IDS_E_NODRIVERFILE);
	}

	m_hScm = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
	if (m_hScm == NULL)
		return MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NULL, IDS_E_SCMOPEN);

	// Create NCapDD Service Entry.
    m_hSrv = CreateServiceW(m_hScm, L"NCapDD", L"Packet Capture",
		SERVICE_ALL_ACCESS,
		SERVICE_KERNEL_DRIVER,
		SERVICE_DEMAND_START,
		SERVICE_ERROR_NORMAL,
		szDriverPath,
		NULL, NULL, NULL, NULL, NULL);
    if (m_hSrv)
	{
		CloseServiceHandle(m_hSrv);
		m_hSrv = NULL;
	}
	else
	{
		long lError = GetLastError();
		if (lError == ERROR_SERVICE_EXISTS) 
		{
			;	// OK
		}
		else
		{
			CloseService();
			return MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NULL, lError);
		}
    }

	m_hSrv = OpenServiceW(m_hScm, L"NCapDD", SERVICE_ALL_ACCESS);
	if (m_hSrv)
	{
		BOOL f = StartService(m_hSrv, 0, NULL);
		if (f == FALSE)
		{
			long lError = GetLastError();
			if (lError != ERROR_SERVICE_ALREADY_RUNNING) 
			{
				CloseService();
				return Error(IDS_E_STARTSERVICE, GUID_NULL, MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NULL, IDS_E_STARTSERVICE));
			}
		}
	}
	else
	{
		CloseService();
		return Error(IDS_E_DRIVEROPEN, GUID_NULL, MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NULL, IDS_E_DRIVEROPEN));
	}

	return S_OK;
}

void CAdapters::ReadMACAddress()
{
	for (int i = 0; i < m_aNIC.size(); i++)
	{
		CAdapterInfo& info = m_aNIC[i];

        WCHAR pszDOSDevName[_MAX_PATH], pszDevName[_MAX_PATH];
		wsprintfW(pszDOSDevName, L"\\\\.\\NCapDD_%s", info.m_sServiceName.c_str());
		wsprintfW(pszDevName, L"%s%s", scDevPrefix, info.m_sServiceName.c_str());
		BOOL Result=DefineDosDeviceW(DDD_RAW_TARGET_PATH, &pszDOSDevName[4], pszDevName);
		if (Result)
		{
			info.m_sDeviceName = pszDOSDevName;
			HANDLE h = CreateFile(pszDOSDevName, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL);
			if (h != INVALID_HANDLE_VALUE)
			{
				DWORD dw;
				DeviceIoControl(h, DIOC_CAPGetMacAddress, NULL, 0, info.m_bMACAddress, 6, &dw, NULL);
				CloseHandle(h);
			}
		}
	}
}

void CAdapters::CloseService()
{
	if (m_hSrv)
		CloseServiceHandle(m_hSrv);
	if (m_hScm)
		CloseServiceHandle(m_hScm);
	m_hScm = m_hSrv = NULL;
}
