// Copyright(c) 1999 arton

#ifndef __MACPACKET_H_
#define __MACPACKET_H_

#include "resource.h"

/////////////////////////////////////////////////////////////////////////////
// CMacPacket
class ATL_NO_VTABLE CMacPacket : 
	public CComObjectRootEx<CComMultiThreadModel>,
	public CComCoClass<CMacPacket, &CLSID_MacPacket>,
	public ISupportErrorInfo,
	public IRawMacPacket,
	public IDispatchImpl<IMacPacketEx, &IID_IMacPacketEx, &LIBID_MACCAPV2Lib, MACCAPLIB_MJ>
{
public:
	CMacPacket() : m_pbData(NULL)
	{
		m_pUnkMarshaler = NULL;
		m_pStream = NULL;
	}

	enum {
		PACKET_HEADER_SIZE = 16,
		RCVTIME_OFFSET = 8,
		//
		MAC_HEADER_SIZE = 14,
		MAC_MAX_SIZE = 1514,	// DST(6) + SRC(6) + TP(2) + DATA(46<=?<=1500)
		IP_HEADER_SIZE = 20,	// minimum size
		// MAC Offset
		DST_MAC_ADDRESS = 0,
		SRC_MAC_ADDRESS = 6,
		ETHER_TYPE = 12,
		// IP Offset
		IPH_VERSION = 14,		// maybe 0x45 or 0x44
		IPH_TOS = 15,
		IPH_LENGTH = 16,
		IPH_TTL = 22,
		IPH_PROTOCOL = 23,
		IPH_SRC = 26,
		IPH_DST = 30,
		// TCP Offset
		TCPH_SRCPORT = 34,
		TCPH_DSTPORT = 36,
		// UDP Offset
		UDPH_SRCPORT = 34,
		UDPH_DSTPORT = 36,
		// ARP
		ARP_HWT = 14,
		ARP_PROTO = 16,
		ARP_HL = 18,
		ARP_PL = 19,
		ARP_OPERATION = 20,
		ARP_SRC_MAC = 22,
		ARP_SRC_IP = 28,
		ARP_DST_MAC = 32,
		ARP_DST_IP = 38,
		ARP_MAX = 42,
	};

DECLARE_REGISTRY_RESOURCEID(IDR_MACPACKET)
DECLARE_NOT_AGGREGATABLE(CMacPacket)
DECLARE_GET_CONTROLLING_UNKNOWN()

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CMacPacket)
	COM_INTERFACE_ENTRY(IMacPacketEx)
	COM_INTERFACE_ENTRY(IMacPacket)
	COM_INTERFACE_ENTRY(IRawMacPacket)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY_AGGREGATE(IID_IMarshal, m_pUnkMarshaler.p)
END_COM_MAP()

	HRESULT FinalConstruct()
	{
		ATLTRACE(_T("Packet Created\n"));
		return CoCreateFreeThreadedMarshaler(
			GetControllingUnknown(), &m_pUnkMarshaler.p);
	}

	void FinalRelease()
	{
		ATLTRACE(_T("Packet Discarded\n"));
		m_pUnkMarshaler.Release();
		if (m_pStream)
			m_pStream->Release();
		if (m_pbData) 
			delete[] m_pbData;
	}

	void SetParam(IStream* pStream, DWORD dwSeekPoint);
	void SetParam(int cb, LPBYTE pb);

	CComPtr<IUnknown> m_pUnkMarshaler;

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// IMacPacket
public:
	HRESULT STDMETHODCALLTYPE get_Value( 
            /* [retval][out] */ VARIANT __RPC_FAR *pData);
        
	HRESULT STDMETHODCALLTYPE get_RecvTime( 
            /* [retval][out] */ DATE __RPC_FAR *pDate);

	HRESULT STDMETHODCALLTYPE get_EtherTypeString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult);
        
	HRESULT STDMETHODCALLTYPE get_IsMulticast( 
            /* [retval][out] */ VARIANT_BOOL __RPC_FAR *pResult);
        
	HRESULT STDMETHODCALLTYPE get_SrcVendorString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult);
        
	HRESULT STDMETHODCALLTYPE get_DestVendorString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult);
        
	HRESULT STDMETHODCALLTYPE get_FileTime( 
            /* [retval][out] */ CY __RPC_FAR *pResult);
        
	HRESULT STDMETHODCALLTYPE get_InternetProtocolString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult);
        
	HRESULT STDMETHODCALLTYPE get_SrcMacString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult);
        
	HRESULT STDMETHODCALLTYPE get_DestMacString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult);
        
	HRESULT STDMETHODCALLTYPE get_SrcIPAddressString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult);
        
	HRESULT STDMETHODCALLTYPE get_DestIPAddressString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult);

	HRESULT STDMETHODCALLTYPE get_EtherType( 
            /* [retval][out] */ long __RPC_FAR *pResult);
        
	HRESULT STDMETHODCALLTYPE get_InternetProtocol( 
            /* [retval][out] */ short __RPC_FAR *pResult);
        
	HRESULT STDMETHODCALLTYPE get_Length( 
            /* [retval][out] */ long __RPC_FAR *pResult);

	HRESULT STDMETHODCALLTYPE get_UnpackedString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult);

	HRESULT STDMETHODCALLTYPE get_AsAnsiString( 
            /* [in] */ long offset,
            /* [in] */ VARIANT length,
            /* [retval][out] */ BSTR __RPC_FAR *pResult);

	HRESULT STDMETHODCALLTYPE get_AsString( 
            /* [in] */ long offset,
            /* [in] */ VARIANT length,
            /* [retval][out] */ BSTR __RPC_FAR *pResult);
// IRawMacPacket
	HRESULT STDMETHODCALLTYPE GetPacket( 
            /* [out] */ long __RPC_FAR *pLength,
            /* [out][in] */ unsigned char __RPC_FAR buff[ 1514 ]);
        
	HRESULT STDMETHODCALLTYPE GetRecvTime( 
            /* [out] */ FILETIME __RPC_FAR *pFileTime);
        
	HRESULT STDMETHODCALLTYPE GetLength( 
            /* [out] */ long __RPC_FAR *pLength);

private:
	IStream* m_pStream;
	LPBYTE m_pbData;
	DWORD m_dwSeekPoint;
	DWORD m_dwSize;
	FILETIME m_ftime;
	HRESULT SetHeader();
	void GetIPAddress(int, BSTR*);
	HRESULT GetLength(VARIANT, int, int&);
};

#endif //__MACPACKET_H_
