// Copyright(c) 1999 arton
#include "stdafx.h"
#include "MacCapV2.h"
#include "PacketCollection.h"
#include "MacPacket.h"

class CPacketEnum : public IEnumVARIANT
{
public:
	CPacketEnum(IStream* pStm, DWORD dwSize, DWORD dwOffset = 0)
		: m_ulRef(1), m_dwSize(dwSize), m_dwCurrent(dwOffset)
	{
		pStm->Clone(&m_pStm);
	}
	~CPacketEnum()
	{
		m_pStm->Release();
	}
	STDMETHOD_(ULONG, AddRef)()
	{
		return ++m_ulRef;
	}
	STDMETHOD_(ULONG, Release)()
	{
		m_ulRef--;
		if (m_ulRef <= 0)
		{
			delete this;
			return 0;
		}
		return m_ulRef;
	}
	STDMETHOD(QueryInterface)(REFIID iid, void ** ppvObject)
	{
		if (!ppvObject) return E_POINTER;
		if (IsEqualIID(IID_IUnknown, iid) 
			|| IsEqualIID(IID_IEnumVARIANT, iid)) {
			*ppvObject = this;
			m_ulRef++;
			return S_OK;
		}
		return E_NOINTERFACE;
	}

    HRESULT STDMETHODCALLTYPE Next(
		ULONG cElements, 
		VARIANT FAR* pvar, 
		ULONG FAR* pcElementFetched)
	{
		if (m_dwCurrent >= m_dwSize) return S_FALSE;

		ULONG l2;
		if (pcElementFetched != NULL)        
			*pcElementFetched = 0;
		for (ULONG l=0; l<cElements; l++)        
			VariantInit(&pvar[l]);
		// Retrieve the next cElements elements.
		DWORD dwPacketSize;
		ULONG ul;
		LARGE_INTEGER dlib;
		dlib.HighPart = 0;
		for (l2 = 0; l2 < cElements; l2++)
		{
			dlib.LowPart = m_dwCurrent;
			m_pStm->Seek(dlib, STREAM_SEEK_SET, NULL);
			if (m_pStm->Read(&dwPacketSize, sizeof(DWORD), &ul) == S_OK && ul == sizeof(DWORD))
			{
				if (dwPacketSize < CMacPacket::PACKET_HEADER_SIZE 
					|| dwPacketSize > (CMacPacket::MAC_MAX_SIZE + CMacPacket::PACKET_HEADER_SIZE))
					break;	// Invalid Record.
				CComObject<CMacPacket>* p;
				HRESULT hr = CComObject<CMacPacket>::CreateInstance(&p);
				if (hr != S_OK)
					break;
				m_pStm->AddRef();
				p->SetParam(m_pStm, m_dwCurrent);
				if (p->QueryInterface(IID_IMacPacket, (void**)&pvar[l2].pdispVal) != S_OK)
				{
					p->Release();
					break;
				}
				pvar[l2].vt = VT_DISPATCH;
				m_dwCurrent += dwPacketSize;
			}
			else
				break;
		}    
		// Set count of elements retrieved.
		if (pcElementFetched != NULL)        
			*pcElementFetched = l2;
		return  (l2 < cElements) ? ResultFromScode(S_FALSE) : ResultFromScode(S_OK);
	}
    HRESULT STDMETHODCALLTYPE Skip(unsigned long celt)
	{
		if (m_dwCurrent >= m_dwSize) return S_FALSE;
		DWORD dwPacketSize;
		ULONG ul;
		LARGE_INTEGER dlib;
		dlib.HighPart = 0;
		while (celt)
		{
			celt--;
			dlib.LowPart = m_dwCurrent;
			m_pStm->Seek(dlib, STREAM_SEEK_SET, NULL);
			if (m_pStm->Read(&dwPacketSize, sizeof(DWORD), &ul) == S_OK && ul == sizeof(DWORD))
			{
				if (dwPacketSize == 0 || dwPacketSize > CMacPacket::MAC_MAX_SIZE)
					return S_FALSE;
				m_dwCurrent += dwPacketSize;
			}
			else
				return S_FALSE;
		}
		return S_OK;
	}
    HRESULT STDMETHODCALLTYPE Reset()
	{
		m_dwCurrent = 0;
		return S_OK;
	}
    HRESULT STDMETHODCALLTYPE Clone(IEnumVARIANT FAR* FAR* ppenum)
	{
		if (!ppenum) return E_POINTER;
		*ppenum = new CPacketEnum(m_pStm, m_dwSize, m_dwCurrent);
		return S_OK;
	}
private:
	ULONG m_ulRef;
	IStream* m_pStm;
	DWORD m_dwCurrent, m_dwSize;
};

/////////////////////////////////////////////////////////////////////////////
// CPacketCollection

HRESULT STDMETHODCALLTYPE CPacketCollection::get__NewEnum( 
            /* [retval][out] */ IUnknown __RPC_FAR *__RPC_FAR *ppUnk)
{
	if (!ppUnk) return E_POINTER;
	STATSTG statstg;
	ZeroMemory(&statstg, sizeof(STATSTG));
	m_pStm->Stat(&statstg, STATFLAG_NONAME);
	*ppUnk = new CPacketEnum(m_pStm, statstg.cbSize.LowPart);
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CPacketCollection::get_Length( 
            /* [retval][out] */ long __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	if (m_cbPackets < 0)
	{
		ULONG ul;
		DWORD dwPacketSize;
		LARGE_INTEGER dlib;
		dlib.HighPart = dlib.LowPart = 0;
		m_pStm->Seek(dlib, STREAM_SEEK_SET, NULL);
		m_cbPackets = 0;
		for (;;)
		{
			if (m_pStm->Read(&dwPacketSize, sizeof(DWORD), &ul) == S_OK && ul == sizeof(DWORD))
			{
				if (dwPacketSize < CMacPacket::PACKET_HEADER_SIZE 
					|| dwPacketSize > (CMacPacket::MAC_MAX_SIZE + CMacPacket::PACKET_HEADER_SIZE))
					break;	// Invalid Record.
				else
				{
					m_cbPackets++;
					dlib.LowPart += dwPacketSize;
					m_pStm->Seek(dlib, STREAM_SEEK_SET, NULL);
				}
			}
			else
				break;
		}
	}			
	*pResult = m_cbPackets;
	return S_OK;
}

